/// <reference types="node" />
import zlib from 'zlib';
import { Compression } from './Compression';
import { CompressOptions, BrotliOptions } from '../interfaces';
import { CompressionExtensions, CompressionNames } from '../enums';
/**
 * Brotli compression
 */
export declare class BrotliCompression extends Compression<BrotliOptions> {
    readonly compressionName = CompressionNames.BROTLI;
    readonly ext = CompressionExtensions.BROTLI;
    /**
     * Creates an instance of BrotliCompression
     */
    constructor(options: CompressOptions);
    /**
     * Returns brotli compression instance in closure.
     */
    getCompression(): zlib.BrotliCompress;
    /**
     * Returns human-readable brotli compression options info.
     */
    readableOptions(): string;
    /**
     * Build brotli options object [compressionOptions].
     */
    protected selectCompression(): void;
    /**
     * Returns human-readable brotli option name.
     */
    protected getBrotliOptionName(index: string): string | undefined;
}
